#!/usr/bin/perl
use strict;
use warnings;

# This script calculates a Kendall's tau correlation score between WMT10 
# human relative rankings and automatic metric scores.  It counts discordant 
# and discordant pairs of human rankings and automatic metric scores and 
# derives a tau value by subtracting discordant from concordant pairs and 
# dividing by total number of pairs.  Pairs with a tie in the human 
# ranking are excluded, as are pairs that involve a no-rank judgment and 
# pairs that involve a judgment on the reference translation as a system.
# Usage:
# perl wmt10_segTau.pl <metric type> <segment-level language-pair specific 
# metric score file> <segment-level language-pair-specific human rank file>
# metric score file>
# Metric type must be either "acc" (accuracy) or "err" (error).  Only TERp 
# is an error-based metric in this distribution.
# Human rank file must be a language-pair-specific .csv file of the kind 
# included with this distribution for WMT10 human ranks.
# Metric score files must be formatted as those included with this 
# distribution, following MetricsMaTr10 segment-level score file format, with 
# all segment scores for one metric and one language pair in one file.  
# E.g.:
# perl wmt_segTau.pl acc Spanish-English-NIST-c-seg.scr 
# data_RNK_Spanish-English.csv
# The resulting tau value is written to standard output.

my $metricType = $ARGV[0];
my %metricScores;

open METRICSCORES, '<', $ARGV[1];

while (<METRICSCORES>) {
    if (/.*?\t(.*?)\t(.*?)\t(.*?)\t(.*?)$/) {
	my $id = $2 . "-" . $3;
# get metric score by doc and sys id
	$metricScores{$id}{$1} = $4;
    }
}
close METRICSCORES;

my $ctConcordPairs = 0;
my $ctDiscordPairs = 0;

open HUMANSCORES, '<', $ARGV[2];
while (<HUMANSCORES>) {
# to avoid header lines, only match if 3rd item (srcindex) is numerical
    if (/(.*?),(.*?),(\d+),(.*?),(.*?),(.*?),(.*?),(.*?),(.*?),(.*?),(.*?),(.*?),(.*?),(.*?),(.*?),(.*?),(-?\d),(-?\d),(-?\d),(-?\d),(-?\d)/) {
	my $id = $4 . "-" . $5;
	push (my @systems, $8, $10, $12, $14, $16);
	push (my @humanScores, $17, $18, $19, $20, $21);
	for (my $i=0; $i < @systems; $i++) {
	    for (my $j = $i+1; $j < @systems; $j++) {
# don't include refs, human no-ranks (-1), human ties
		if ($systems[$i] ne "_ref" && $systems[$j] ne "_ref" && $humanScores[$i] != -1 && $humanScores[$j] != -1 && $humanScores[$i] != $humanScores[$j]) {
		    my $sysA = $systems[$i];
		    my $humanScoreA = $humanScores[$i];
		    my $metricScoreA = $metricScores{$id}{$sysA};
		    my $sysB = $systems[$j];
		    my $humanScoreB = $humanScores[$j];
		    my $metricScoreB = $metricScores{$id}{$sysB};
# error metrics (better translation = lower human ranking value, lower 
# automatic metric score)
		    if ($metricType eq "err") {
			if ($humanScoreA > $humanScoreB && $metricScoreA > $metricScoreB) {
			    $ctConcordPairs++;
			} elsif ($humanScoreA < $humanScoreB && $metricScoreA < $metricScoreB) {
			    $ctConcordPairs++;
			} else {
			    $ctDiscordPairs++;
			}
		    } else {
# accuracy metrics (better translation = lower human ranking value, higher 
# automatic metric score)
			if ($humanScoreA > $humanScoreB && $metricScoreA < $metricScoreB) {
			    $ctConcordPairs++;
			} elsif ($humanScoreA < $humanScoreB && $metricScoreA > $metricScoreB) {
			    $ctConcordPairs++;
			} else {
			    $ctDiscordPairs++;
			}
		    }
		}
	    }
	}
    }
}
close HUMANSCORES;

# tau calculation
my $tau = ($ctConcordPairs - $ctDiscordPairs) / ($ctConcordPairs + $ctDiscordPairs);

# make tau value negative for error metrics
if ($metricType eq "err") {
    $tau = "-" . $tau;
}

print "$tau\n";
